% Pronys method to
%   f = function
%   n = estimate for deg R
%   s = #Variables

function [ F,K,LK,N,dK,dN,k ] = Prony1( n,s )
  Frnk = 0;
  k = 0;
  Fnew = FnkFunc( n,k );
  Fnkrnk = rank( Fnew );
  noK = 1;
  K = [];
  LK = [];
  LKm = 0; LKn = 0;
  N = [ ];
  Nm = 0; Nn = 0;

  % degree vectors

  dK = [];
  dN = [];

  while Fnkrnk != Frnk
    printf( "Degree %d, rank %d ...\n", k,Fnkrnk );
    F = Fnew;
    
    % Full and homogeneous degree
    fdeg = bincoeff( s+k,k );
    hdeg = bincoeff( s+k-1,k );

    % kernel of Fnk
    [ Fm,Fn ] = size( F );
    [U,S,V] = svd( F );

    %disp ( F );
    %disp ( diag(S) );

    % There is a kernel ...
    if Fnkrnk < Fn
      Pk = V( :,Fnkrnk+1:Fn );
      [ Pkm,Pkn ] = size( Pk );
      % Add Pk to kernel
      if noK == 1
	 % The first time
	 K = Pk;
	 Kadd = Pk;
	 noK = 0;
	 vk = Pkn;
      else
	  % Real extension
	  [ Km,Kn ] = size( K );
	  X = Pk \ [ K; zeros( Pkm-Km,Kn ) ];
	  [ QX,RX ] = qr( X );
	  Kadd = Pk * QX( :,vk+1:Pkn );
	  K = [ [ K; zeros( Pkm-Km,Kn ) ], Kadd ];
	  vk = Pkn;
      end
      dK = [ dK, k*ones( 1,size(Kadd)(2) ) ];

      % Extend leading terms
      LPk = Kadd( fdeg-hdeg+1:fdeg, : );
      [ LPkm, LPkn ] = size( LPk );
      LK = [ [ LK; zeros( fdeg-LKm,LKn ) ],[ zeros( fdeg-hdeg,LPkn ); LPk ] ]; 
      [ LKm,LKn ] = size( LK );

      % Extend Quotient space
      [ QP,RP ] = qr( LPk );
      Nadd = QP( :,LPkn+1:LPkm );
      [ Naddm,Naddn ] = size( Nadd );
      N = [ [ N; zeros( Naddm,Nn ) ], [ zeros( Nm,Naddn); Nadd ] ];
      [ Nm, Nn ] = size( N );
      dN = [ dN, k*ones( 1,Naddn ) ];
    else
    % All goes into quotient space
      N = [ [ N; zeros( hdeg,Nn ) ], [ zeros( Nm,hdeg); eye( hdeg ) ] ];
      [ Nm, Nn ] = size( N );
      dN = [ dN, k*ones( 1,hdeg ) ];
    end

    % Next one, loop on
    Frnk = Fnkrnk;
    k = k+1;
    Fnew = FnkFunc( n,k );
    Fnkrnk = rank( Fnew );	
  end

  % Complete H-basis

  printf( "Finalizing degree %d ...\n", k );
  F = Fnew;
  
  % Full and homogeneous degree
  fdeg = bincoeff( s+k,k );
  hdeg = bincoeff( s+k-1,k );
  
  % kernel of Fnk
  [ Fm,Fn ] = size( F );
  [U,S,V] = svd( F );
  
  %disp ( diag(S) );

  % There is a kernel ...
  if Fnkrnk < Fn
    Pk = V( :,Fnkrnk+1:Fn );
    [ Pkm,Pkn ] = size( Pk );
    % Add Pk to kernel
    if noK == 1
      % The first time
      K = Pk;
      Kadd = Pk;
      noK = 0;
      vk = Pkn;
    else
      % Real extension
      [ Km,Kn ] = size( K );
      X = Pk \ [ K; zeros( Pkm-Km,Kn ) ];
      [ QX,RX ] = qr( X );
      Kadd = Pk * QX( :,vk+1:Pkn );
      K = [ [ K; zeros( Pkm-Km,Kn ) ], Kadd ];
      vk = Pkn;
    end
    dK = [ dK, k*ones( 1,size(Kadd)(2) ) ];

    LPk = Kadd( fdeg-hdeg+1:fdeg, : );
    [ LPkm, LPkn ] = size( LPk );
    LK = [ [ LK; zeros( fdeg-LKm,LKn ) ], [ zeros(  fdeg-hdeg,LPkn ); LPk ] ]; 
    [ LKm,LKn ] = size( LK );
  end

  % Done ...
end
